% ssmake3MBf - the full-matrix version of the triple 'make' file to end (hopefully) all triple make files.
%
% Generates 4 systems of state-space matrices representing a triple pendulum.
%
% This file loads file triple.m which should define structure pend with fields for
%  all required parameters.
%
% Depending on the selected options, this file loads one of symbexport3.m,
%  symbexport3db.m, symbexport3lat.m, or symbexport3dblat.m. These contain
%  mechanically generated Matlab code for state-space matrix elements
%  exported from the corresponding Mathematica model by Mark Barton.
%
% If pend.db is defined and non-zero, top mass has dual blades at both levels 
%  as proposed for the AdvLIGO beamsplitter, otherwise, quad blades at top
%  mass a la GEO.
%
% If pend.kx1 and pend.kx2 are defined as per-side stiffnesses, blades can flex
%  laterally, otherwise just vertically.
%
% If pend.ribbon is defined and non-zero, bottom stage is taken to have
%  ribbons of thickness pend.t3 and width pend.W3.
%
% If pend.stage2 is defined and non-zero, d0-d4 are interpreted as raw
%  values, i.e., as actual wire breakoff vertical positions, and fudges are
%  applied to include the effect of wire flexure. The fudges used give essentially
%  perfect agreement with the output of Stage 2 of the equivalent
%  Mathematica model if pend.ribbon is not set, and very good agreement
%  otherwise. If pend.stage2 is undefined or 0, d0-d4 are taken as effective values, 
%  and flexure correction is assumed to be inapplicable (as for infinitely 
%  flexible wire) or to have been applied manually.
%
% Ultimately based on the mathematical model of Calum Iain Torrie et al. 1998
% with contributions from K.A. Strain 6/99.

% 8/16/06: initial version based on ssmake3pv2nMB2.m for GEO-style triple with 
%  four blades, with lots of improvements cut'n'pasted from ssm4pv2eMB4.m.
%  Transverse/roll SS object renamed from rtn to trn to avoid naming
%  conflict with Real-Time Workshop. All B matrix signs hand-checked.

% 3/4/07: corrected formula for flex1/2/3 (was '4' instead of 'nw1'/'nw2'/'nw3')

% 6/21/2011: renamed ssmake3MBf.m, reworked to use full matrices from symbexport3full.m etc.

% load in the pendulum parameters
global pend;
triplep;

% Set any undefined flags to default values
if not(isfield(pend,'stage2'))
    pend.stage2=0; % define, default to false (wire flexure correction neglected or implicit)
end
if not(isfield(pend,'ribbon') )
    pend.ribbon=0; % define, default to false (round fibres)
end
if not(isfield(pend,'db') )
    pend.db=0; % define, default to false (GEO-style quad blades, not AdvLIGO-style double)
end
if not(isfield(pend,'g') )
    pend.g=9.81; 
end

% general redefintion of variables to simplify notation
% and cope with different pendulum designs

g = pend.g;
Y1   = pend.Y1;
Y2   = pend.Y2;
Y3   = pend.Y3;

% Allow for blade stiffnesses to be specified directly rather than via ufcs.
if isfield(pend, 'kc1')
    kc1 = pend.kc1;
    pend.ufc1 = sqrt(2*kc1/pend.m1)/2/pi;
else
    kc1	=  1/2 * (2*pi*pend.ufc1)^2*pend.m1;
    pend.kc1 = kc1;
end
if isfield(pend, 'kc2')
    kc2 = pend.kc2;
    pend.ufc2 = sqrt(2*kc2/pend.m2)/2/pi;
else
    kc2	=  1/2 * (2*pi*pend.ufc2)^2*pend.m2;
    pend.kc2 = kc2;
end

l1  =pend.l1;
l2  =pend.l2;
l3  =pend.l3;

% wire stiffnesses (per side, as opposed to per wire for the Mathematica)
kw1	=pend.Y1*pi*pend.r1^2/l1*pend.nw1/2;	
kw2	=pend.Y2*pi*pend.r2^2/l2*pend.nw2/2;	
if not(isfield(pend,'kw3')) % vertical stiffness of fibers
    if pend.ribbon
        kw3	= pend.Y3*pend.W3*pend.t3/l3*pend.nw3/2;
    else
        kw3	= pend.Y3*pi*pend.r3^2/l3*pend.nw3/2;	
    end
else
    kw3 = pend.kw3;
end

%********************************************************************

% allows choice of 2 wires to set separation to zero

s0=pend.su;
s1=pend.su;
s2=pend.si;
s3=pend.si;
s4=pend.sl;
s5=pend.sl;

d0=pend.d0;
d1=pend.d1;
d2=pend.d2;
d3=pend.d3;
d4=pend.d4;

%********************************************************************

m1 = pend.m1;
m2 = pend.m2;
m3 = pend.m3;

I1x = pend.I1x;
I2x = pend.I2x;
I3x = pend.I3x;
I1y = pend.I1y;
I2y = pend.I2y;
I3y = pend.I3y;
I1z = pend.I1z;
I2z = pend.I2z;
I3z = pend.I3z;

% for backward compatibility, default off-axis MOI terms to zero
if isfield(pend, 'I1xy')
else
    pend.I1xy = 0;
end
if isfield(pend, 'I1yz')
else
    pend.I1yz = 0;
end
if isfield(pend, 'I1zx')
else
    pend.I1zx = 0;
end
if isfield(pend, 'I2xy')
else
    pend.I2xy = 0;
end
if isfield(pend, 'I2yz')
else
    pend.I2yz = 0;
end
if isfield(pend, 'I2zx')
else
    pend.I2zx = 0;
end
if isfield(pend, 'I3xy')
else
    pend.I3xy = 0;
end
if isfield(pend, 'I3yz')
else
    pend.I3yz = 0;
end
if isfield(pend, 'I3zx')
else
    pend.I3zx = 0;
end

I1xy = pend.I1xy;
I1yz = pend.I1yz;
I1zx = pend.I1zx;
I2xy = pend.I2xy;
I2yz = pend.I2yz;
I2zx = pend.I2zx;
I3xy = pend.I3xy;
I3yz = pend.I3yz;
I3zx = pend.I3zx;
m13	= m1+m2+m3;
m23	= m2+m3;

n0  = pend.n0;
n1  = pend.n1;
n2  = pend.n2;
n3  = pend.n3;
n4  = pend.n4;
n5  = pend.n5;

%***********************************************************************************

% cosine and sine of the angle the wire makes with the vertical (z)

si1=(n1-n0)/l1;			%sin(omega1)
si2=(n3-n2)/l2;			%sin(omega2)
si3=(n5-n4)/l3;			%sin(omega3)	
	
c1=(l1^2-(n1-n0)^2)^0.5/l1;	%cos(omega1)
c2=(l2^2-(n3-n2)^2)^0.5/l2;	%cos(omega2)
c3=(l3^2-(n5-n4)^2)^0.5/l3;	%cos(omega3)

k1 = kw1;
k2 = kw2;
k3 = kw3;
su=pend.su;
si=pend.si;
sl=pend.sl;

% Vertical distances between centres of mass 
pend.tl1 = sqrt(pend.l1^2 - (pend.n0-pend.n1)^2) + pend.d0;
pend.tl2 = sqrt(pend.l2^2 - (pend.n2-pend.n3)^2) + pend.d1 + pend.d2;
pend.tl3 = sqrt(pend.l3^2 - (pend.n4-pend.n5)^2) + pend.d3 + pend.d4;

% Distance to the centre of mass from suspension point 
pend.l_suspoint_to_centreofoptic  =  pend.tl1+pend.tl2+pend.tl3;

% Distance to the bottom of the test mass from suspension point 
pend.l_suspoint_to_bottomofoptic  =  pend.tl1+pend.tl2+pend.tl3+pend.tr;

% Stiffness of fibres/ribbons
pi = 4*atan(1);
M11 = (1/4)*pi*pend.r1^4;
M21 = (1/4)*pi*pend.r2^4;
if pend.ribbon
    M31 = (1/4)*pend.W3*pend.t3^3/12;
    M32 = (1/4)*pend.W3^3*pend.t3/12;
else
    M31 = (1/4)*pi*pend.r3^4;
    M32 = (1/4)*pi*pend.r3^4;
end
pend.flex1 = sqrt(nw1*M11*Y1/m13/g)*c1^(3/2);
pend.flex2 = sqrt(nw2*M21*Y2/m23/g)*c2^(3/2);
pend.flex3 = sqrt(nw3*M31*Y3/m3/g)*c3^(3/2); % for long/pitch, yaw and vertical
pend.flex3tr = sqrt(nw3*M32*Y3/m3/g)*c3^(3/2); % for trans/roll, with M32 instead of M31

flex1 = pend.flex1;
flex2 = pend.flex2;
flex3 = pend.flex3; % for long/pitch, yaw and vertical; see below for trans/roll

if pend.stage2
    % apply fudges to approximate Mathematica Stage2 results
    d0 = d0 + flex1; 
    d1 = d1 + flex2;
    d2 = d2 + flex2;
    d3 = d3 + flex3;
    d4 = d4 + flex3;
    l1 = l1 - 2*flex1/c1;
    l2 = l2 - 2*flex2/c2;
    l3 = l3 - 2*flex3/c3;
    n0 = n0 + si1*flex1/c1;
    n1 = n1 - si1*flex1/c1;
    n2 = n2 + si2*flex2/c2;
    n3 = n3 - si2*flex2/c2;
    n4 = n4 + si3*flex3/c3;
    n5 = n5 - si3*flex3/c3;
    disp('Using Stage 2 fudges');
else
    disp('Not using Stage 2 fudges');
end

latelements=0; 
if isfield(pend,'kx1') 
    if isfield(pend,'kx2')
        % lateral compliances have been defined - use appropriate matrix elements
        disp('Using matrix elements with blade lateral compliance');
        latelements = 1;
        kx1 = pend.kx1;
        kx2 = pend.kx2;
        if pend.db
            symbexport3dblatfull
        disp('Using matrix elements with double blades at top mass');
        else
         disp('Using matrix elements with quad blades at top mass');
           symbexport3latfull
        end
    else

    end   
end

if not(latelements)
    disp('Using matrix elements with no blade lateral compliance');
	if pend.db
        disp('Using matrix elements with double blades at top mass');
        symbexport3dbfull
    else
        disp('Using matrix elements with quad blades at top mass');
        symbexport3full
    end
end

mbtripleA = [...
    zeros(18) eye(18)
    (-km)\(xm-cqxm'/qm*cqxm) -bd*eye(18)
];

bm = (-km)\(cxsm-cqxm'/qm*cqsm); % ground displacement inputs: x,y,z,yaw,pitch,roll
bm2 = km\eye(18); % coordinate force inputs: pitchn,xn,pitch1,x1,...

mbtripleB = [...
    zeros(18,24)
    bm bm2
];

mbtripleC = [...
    eye(18) zeros(18,18)
];
 
 mbtripleD = [...
    zeros(18,24)
 ];

mbtriple = ss(mbtripleA,mbtripleB,mbtripleC,mbtripleD);

