% ssmake3MB - the triple 'make' file to end (hopefully) all triple make files.
%
% Generates 4 systems of state-space matrices representing a triple pendulum.
%
% This file loads file triple.m which should define structure pend with fields for
%  all required parameters.
%
% Depending on the selected options, this file loads one of symbexport3.m,
%  symbexport3db.m, symbexport3lat.m, or symbexport3dblat.m. These contain
%  mechanically generated Matlab code for state-space matrix elements
%  exported from the corresponding Mathematica model by Mark Barton.
%
% If pend.db is defined and non-zero, top mass has dual blades at both levels 
%  as proposed for the AdvLIGO beamsplitter, otherwise, quad blades at top
%  mass a la GEO.
%
% If pend.kx1 and pend.kx2 are defined as per-side stiffnesses, blades can flex
%  laterally, otherwise just vertically.
%
% If pend.ribbon is defined and non-zero, bottom stage is taken to have
%  ribbons of thickness pend.t3 and width pend.W3.
%
% If pend.stage2 is defined and non-zero, d0-d4 are interpreted as raw
%  values, i.e., as actual wire breakoff vertical positions, and fudges are
%  applied to include the effect of wire flexure. The fudges used give essentially
%  perfect agreement with the output of Stage 2 of the equivalent
%  Mathematica model if pend.ribbon is not set, and very good agreement
%  otherwise. If pend.stage2 is undefined or 0, d0-d4 are taken as effective values, 
%  and flexure correction is assumed to be inapplicable (as for infinitely 
%  flexible wire) or to have been applied manually.
%
% Ultimately based on the mathematical model of Calum Iain Torrie et al. 1998
% with contributions from K.A. Strain 6/99.

% 8/16/06: initial version based on ssmake3pv2nMB2.m for GEO-style triple with 
%  four blades, with lots of improvements cut'n'pasted from ssm4pv2eMB4.m.
%  Transverse/roll SS object renamed from rtn to trn to avoid naming
%  conflict with Real-Time Workshop. All B matrix signs hand-checked.

% 3/4/07: corrected formula for flex1/2/3 (was '4' instead of 'nw1'/'nw2'/'nw3')

% load in the pendulum parameters
global pend;
doublep;

% Set any undefined flags to default values
if not(isfield(pend,'stage2'))
    pend.stage2=0; % define, default to false (wire flexure correction neglected or implicit)
end
if not(isfield(pend,'ribbon') )
    pend.ribbon=0; % define, default to false (round fibres)
end
if not(isfield(pend,'bd') )
    pend.bd=0; % define, default to 0
end
if not(isfield(pend,'g') )
    pend.g=9.81; 
end
if isfield(pend,'si')
    pend.sl=pend.si; 
end

% general redefintion of variables to simplify notation
% and cope with different pendulum designs

g = pend.g;
Y1   = pend.Y1;
Y2   = pend.Y2;
bd   = pend.bd;

% Allow for blade stiffnesses to be specified directly rather than via ufcs.
if isfield(pend, 'kc1')
    kc1 = pend.kc1;
    pend.ufc1 = sqrt(2*kc1/pend.m1)/2/pi;
else
    kc1	=  1/2 * (2*pi*pend.ufc1)^2*pend.m1;
    pend.kc1 = kc1;
end
if isfield(pend, 'kc2')
    kc2 = pend.kc2;
    pend.ufc2 = sqrt(2*kc2/pend.m2)/2/pi;
else
    kc2	=  1/2 * (2*pi*pend.ufc2)^2*pend.m2;
    pend.kc2 = kc2;
end

l1  =pend.l1;
l2  =pend.l2;

% wire stiffnesses (per side, as opposed to per wire for the Mathematica)
kw1	=pend.Y1*pi*pend.r1^2/l1*pend.nw1/2;	
kw2	=pend.Y2*pi*pend.r2^2/l2*pend.nw2/2;	

%********************************************************************

% allows choice of 2 wires to set separation to zero

s0=pend.su;
s1=pend.su;
s2=pend.si;
s3=pend.si;

d0=pend.d0;
d1=pend.d1;
d2=pend.d2;

%********************************************************************

m1 = pend.m1;
m2 = pend.m2;

I1x = pend.I1x;
I2x = pend.I2x;
I1y = pend.I1y;
I2y = pend.I2y;
I1z = pend.I1z;
I2z = pend.I2z;

m12	= m1+m2;

n0  = pend.n0;
n1  = pend.n1;
n2  = pend.n2;
n3  = pend.n3;

%***********************************************************************************

% cosine and sine of the angle the wire makes with the vertical (z)

si1=(n1-n0)/l1;			%sin(omega1)
si2=(n3-n2)/l2;			%sin(omega2)
	
c1=(l1^2-(n1-n0)^2)^0.5/l1;	%cos(omega1)
c2=(l2^2-(n3-n2)^2)^0.5/l2;	%cos(omega2)

k1 = kw1;
k2 = kw2;
su=pend.su;
si=pend.si;
sl=pend.si;

nw1=pend.nw1;
nw2=pend.nw2;

% Vertical distances between centres of mass 
pend.tl1 = sqrt(pend.l1^2 - (pend.n0-pend.n1)^2) + pend.d0;
pend.tl2 = sqrt(pend.l2^2 - (pend.n2-pend.n3)^2) + pend.d1 + pend.d2;

% Stiffness of fibres/ribbons
pi = 4*atan(1);
M11 = (1/4)*pi*pend.r1^4;
M21 = (1/4)*pi*pend.r2^4;
pend.flex1 = sqrt(nw1*M11*Y1/m12/g)*c1^(3/2);
pend.flex2 = sqrt(nw2*M21*Y2/m2/g)*c2^(3/2);

flex1 = pend.flex1;
flex2 = pend.flex2;

if pend.stage2
    % apply fudges to approximate Mathematica Stage2 results
    d0 = d0 + flex1; 
    d1 = d1 + flex2;
    d2 = d2 + flex2;
    l1 = l1 - 2*flex1/c1;
    l2 = l2 - 2*flex2/c2;
    n0 = n0 + si1*flex1/c1;
    n1 = n1 - si1*flex1/c1;
    n2 = n2 + si2*flex2/c2;
    n3 = n3 - si2*flex2/c2;
    disp('Using Stage 2 fudges');
else
    disp('Not using Stage 2 fudges');
end

latelements=0; 
if isfield(pend,'kx1') 
    if isfield(pend,'kx2')
        % lateral compliances have been defined - use appropriate matrix elements
        disp('Using matrix elements with blade lateral compliance');
        latelements = 1;
        kx1 = pend.kx1;
        kx2 = pend.kx2;
        symbexport2lat;
    else

    end   
end

if not(latelements)
    disp('Using matrix elements with no blade lateral compliance');
    symbexport2;
end

% LONGITUDINAL-PITCH

AAALP = [...
    zeros(4) eye(4)
    (-kmlp\(xmlp-cqxmlp'/qm*cqxmlp)) -bd*eye(4)
];

bmlp = (-kmlp)\(cxsmlp-cqxmlp'/qm*cqsm); % ground displacement inputs: x,y,z,yaw,pitch,roll
bmlp2 = (kmlp)\eye(4); % coordinate force inputs: pitchn,xn,pitch1,x1,...

BBBLP = [...
    zeros(4,7)
    bmlp(:,1) bmlp2(:,2) bmlp2(:,2) bmlp2(:,4) bmlp2(:,1) bmlp2(:,1) bmlp2(:,3)
];

CCCLP = [...
 0 1 0 0 0 0 0 0
 0 0 0 1 0 0 0 0
 0 0 0 0 0 0 0 0
 1 0 0 0 0 0 0 0
 0 0 1 0 0 0 0 0
 0 0 0 0 0 0 0 0
 ];
 
 DDDLP = [...
 0 0 0 0 0 0 0
 0 0 0 0 0 0 0
 0 0 0 0 0 0 0
 0 0 0 0 0 0 0
 0 0 0 0 0 0 0
 0 0 0 0 0 0 0
];

lpe = ss(AAALP,BBBLP,CCCLP,DDDLP);

as = [0];
bs = [0];
cs = [0];
ds = [1];

[a,b,c,d]   = append(as,bs,cs,ds,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[alp,blp,clp,dlp]   = series(a,b,c,d,AAALP,BBBLP,CCCLP,DDDLP);

lpn = ss(alp,blp,clp,dlp);


%YAW

AAAY = [...
    zeros(2) eye(2)
    -kmy\(xmy-cqxmy'/qm*cqxmy) -bd*eye(2)
];

bmy = -kmy\(cxsmy-cqxmy'/qm*cqsm); % ground displacement inputs: x,y,z,yaw,pitch,roll
bmy2 = kmy\eye(2); % coordinate force inputs: yawn,yaw1,...

BBBY = [...
    zeros(2,4)
    bmy(:,4) bmy2(:,1) bmy2(:,1) bmy2(:,2)
];

bmv = -kmv\(cxsmv-cqxmv'/qm*cqsm); % ground displacement inputs: x,y,z,yaw,pitch,roll
bmv2 = kmv\eye(2); % coordinate force inputs: yawn,yaw1,...

CCCY = [...
    1	0	0	0
   	0	1	0	0
   	0	0	0	0
];

DDDY = [
    0 0 0 0
    0 0 0 0
    0 0 0 0
];

ye = ss(AAAY,BBBY,CCCY,DDDY);

[acr,bcr,ccr,dcr]   =  append(as,bs,cs,ds,as,bs,cs,ds);
[acr,bcr,ccr,dcr]   =  append(acr,bcr,ccr,dcr,as,bs,cs,ds);
[acr,bcr,ccr,dcr]   =  append(acr,bcr,ccr,dcr,as,bs,cs,ds);
[ay,by,cy,dy]       = series(acr,bcr,ccr,dcr,AAAY,BBBY,CCCY,DDDY);
yn = ss(ay,by,cy,dy);


   
% VERTICAL
AAAV = [...
    zeros(2) eye(2)
    -kmv\(xmv-cqxmv'/qm*cqxmv) -bd*eye(2)
];

BBBV = [...
    zeros(2,3)
    bmv(:,3) bmv2(:,1) bmv2(:,2)
];

CCCV = [
    1	0	0 0
    0	1	0 0
    0	0	0 0
];
   
DDDV = [
    0 0 0
    0 0 0
    0 0 0
];

ve = ss(AAAV,BBBV,CCCV,DDDV);


[acv,bcv,ccv,dcv]   = append(as,bs,cs,ds,as,bs,cs,ds);
[acv,bcv,ccv,dcv]   = append(acv,bcv,ccv,dcv,as,bs,cs,ds);
[av,bv,cv,dv]   = series(acv,bcv,ccv,dcv,AAAV,BBBV,CCCV,DDDV);
vn = ss(av,bv,cv,dv);

% TRANSVERSE-ROLL
if latelements
       symbexport2lat
else
    symbexport2
end

AAATR = [...
    zeros(4) eye(4)
    -kmtr\(xmtr-cqxmtr'/qm*cqxmtr) -bd*eye(4)
];

bmtr = -kmtr\(cxsmtr-cqxmtr'/qm*cqsm); % ground displacement inputs: x,y,z,yaw,pitch,roll
bmtr2 = kmtr\eye(4); % coordinate force inputs: roll1,y1,roll2,y2,...

BBBTR = [...
    zeros(4,6)
    bmtr(:,2) bmtr2(:,2) bmtr2(:,2)  bmtr(:,6) bmtr2(:,1) bmtr2(:,1)
];

CCCTR = [...
    0	1	0	0	0	0	0	0
    0	0	0	1	0	0	0	0
    0	0 	0	0	0	0	0	0
    1	0	0	0	0	0	0	0
    0	0	1	0	0	0	0	0
    0	0	0	0	0   0	0	0
];
 
DDDTR = [...
    0   0  0  0  0  0  
    0   0  0  0  0  0
    0   0  0  0  0  0
    0   0  0  0  0  0
    0   0  0  0  0  0
    0   0  0  0  0  0
];

rte = ss(AAATR,BBBTR,CCCTR,DDDTR);   

[ac,bc,cc,dc]   = append(as,bs,cs,ds,as,bs,cs,ds);
[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);

[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);  %NEW LINE

[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);
[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);
[art,brt,crt,drt]   = series(ac,bc,cc,dc,AAATR,BBBTR,CCCTR,DDDTR);
trn = ss(art,brt,crt,drt);
